package jp.hishidama.javadb.tool;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Properties;

import jp.hishidama.javadb.tool.schema.DerbySchemaFrame;

public class ToolProperties extends Properties {
	private static final long serialVersionUID = 1L;

	public static ToolProperties instance = new ToolProperties();

	public void load(File f) {
		try {
			FileInputStream is = new FileInputStream(f);
			try {
				InputStreamReader r = new InputStreamReader(is,
						DerbySchemaFrame.encoding);
				try {
					super.load(r);
				} finally {
					r.close();
				}
			} finally {
				is.close();
			}
		} catch (IOException e) {
			System.err.println(e.getMessage());
		}

		if (getDriverJarPath() == null) {
			String path = "C:\\Program Files\\Sun\\JavaDB\\lib\\derby.jar"
					+ File.pathSeparator
					+ "C:\\Program Files\\Sun\\JavaDB\\lib\\derbyclient.jar"
					+ File.pathSeparator
					+ "C:\\Program Files\\Java\\jdk1.6.0\\db\\lib\\derby.jar"
					+ File.pathSeparator
					+ "C:\\Program Files\\Java\\jdk1.6.0\\db\\lib\\derbyclient.jar";
			setDriverJarPath(path);
		}

		if (getDateFormat() == null) {
			setDateFormat("yyyy-MM-dd", null);
		}
		if (getTimeFormat() == null) {
			setTimeFormat("HH:mm:ss", null);
		}
		if (getTimestampFormat() == null) {
			setTimestampFormat("yyyy-MM-dd HH:mm:ss", null);
		}
	}

	public void store(File f) throws IOException {
		FileOutputStream os = new FileOutputStream(f);
		try {
			OutputStreamWriter w = new OutputStreamWriter(os,
					DerbySchemaFrame.encoding);
			try {
				super.store(w, "HtJavaDBtool");
			} finally {
				w.close();
			}
		} finally {
			os.close();
		}
	}

	private static final String DBLIST_INDEX = "dblist.index";

	public void setDblistIndex(int n) {
		setProperty(DBLIST_INDEX, Integer.toString(n));
	}

	public int getDblistIndex() {
		String s = getProperty(DBLIST_INDEX);
		try {
			return Integer.parseInt(s);
		} catch (Exception e) {
			return -1;
		}
	}

	private static final String DRIVER_INDEX = "Driver.radiogroup.index";
	private static final String DRIVER_JAR = "Driver.jar.path";

	public void setDriverIndex(int n) {
		setProperty(DRIVER_INDEX, Integer.toString(n));
	}

	public int getDriverIndex() {
		String s = getProperty(DRIVER_INDEX);
		try {
			int n = Integer.parseInt(s);
			if (0 <= n && n <= 1) {
				return n;
			}
		} catch (Exception e) {
		}
		return 1;
	}

	public void setDriverJarPath(String s) {
		setProperty(DRIVER_JAR, s);
	}

	public String getDriverJarPath() {
		return getProperty(DRIVER_JAR);
	}

	private static final String DATE_FORMAT = "Date.format";
	private static final String TIME_FORMAT = "Time.format";
	private static final String TIMESTAMP_FORMAT = "Timestamp.format";

	private SimpleDateFormat dateDF, timeDF, tstampDF;

	public void setDateFormat(String value, SimpleDateFormat f) {
		dateDF = f;
		setProperty(DATE_FORMAT, value);
	}

	public String getDateFormat() {
		return getProperty(DATE_FORMAT);
	}

	public SimpleDateFormat getDateDF() {
		if (dateDF == null) {
			dateDF = new SimpleDateFormat(getDateFormat());
		}
		return dateDF;
	}

	public void setTimeFormat(String value, SimpleDateFormat f) {
		timeDF = f;
		setProperty(TIME_FORMAT, value);
	}

	public String getTimeFormat() {
		return getProperty(TIME_FORMAT);
	}

	public SimpleDateFormat getTimeDF() {
		if (timeDF == null) {
			timeDF = new SimpleDateFormat(getTimeFormat());
		}
		return timeDF;
	}

	public void setTimestampFormat(String value, SimpleDateFormat f) {
		tstampDF = f;
		setProperty(TIMESTAMP_FORMAT, value);
	}

	public String getTimestampFormat() {
		return getProperty(TIMESTAMP_FORMAT);
	}

	public SimpleDateFormat getTimestampDF() {
		if (tstampDF == null) {
			tstampDF = new SimpleDateFormat(getTimestampFormat());
		}
		return tstampDF;
	}
}
