package jp.hishidama.javadb.tool.conn;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import sun.misc.Service;

import jp.hishidama.javadb.tool.ToolProperties;
import jp.hishidama.javadb.tool.free.FreeSqlFrame;
import jp.hishidama.javadb.tool.schema.DerbySchemaFrame;

public class ConnManager {

	private static DelegateConnection connection = new DelegateConnection();
	private static Driver useDriver = null;
	private static String connectUrl = null;

	static List<Driver> driverList = new ArrayList<Driver>();

	static ClassLoader defaultLoader = null;
	static URLClassLoader myLoader = null;

	static String oldJarPath;

	public static synchronized void openConnection(String url) {
		try {
			closeConnection();

			Connection conn;
			ToolProperties prop = ToolProperties.instance;
			if (prop.getDriverIndex() == 0) {
				DerbySchemaFrame.logln("DriverManager.getConnection(" + url
						+ ")");
				conn = DriverManager.getConnection(url);
				useDriver = null;
			} else {
				DerbySchemaFrame.logln("Driver#connect(" + url + ")");
				conn = connect(url, null);
			}
			connection.setConnection(conn);
			connection.setAutoCommit(false);
			connectUrl = url;
		} catch (RuntimeException e) {
			DerbySchemaFrame.log(e);
			throw e;
		} catch (Exception e) {
			DerbySchemaFrame.log(e);
			throw new RuntimeException(e);
		}

		setConnectionActionEnabled(true);
	}

	public static synchronized void closeConnection() {
		if (connection.conn != null) {
			try {
				connection.rollback();
				connection.close();
			} catch (SQLException e) {
				DerbySchemaFrame.log(e);
				throw new RuntimeException(e);
			}
			connection.conn = null;

			try {
				shutdown(false);
			} catch (SQLException e) {
				DerbySchemaFrame.log(e);
			}
		}

		setConnectionActionEnabled(false);
	}

	public static synchronized void closeConnectionForce() {
		if (connection.conn != null) {
			try {
				connection.rollback();
			} catch (SQLException e) {
				DerbySchemaFrame.log(e);
			}
			try {
				connection.close();
			} catch (SQLException e) {
				DerbySchemaFrame.log(e);
			}
			connection.conn = null;

			try {
				shutdown(true);
			} catch (SQLException e) {
				DerbySchemaFrame.log(e);
			}
		}

		setConnectionActionEnabled(false);
	}

	protected static void setConnectionActionEnabled(boolean b) {
		DerbySchemaFrame.instance.actionDisconnect.setEnabled(b);
		FreeSqlFrame.actionOpen.setEnabled(b);
	}

	public static Connection getConnection() {
		if (connection.conn == null) {
			throw new NullPointerException("connection closed");
		}
		return connection;
	}

	static synchronized Connection connect(String url, Properties info)
			throws SQLException, MalformedURLException, InstantiationException,
			IllegalAccessException, ClassNotFoundException {

		ToolProperties prop = ToolProperties.instance;
		String path = prop.getDriverJarPath();
		if (path == null) {
			path = "";
		}
		if (oldJarPath == null || !oldJarPath.equals(path)) {
			driverList.clear();
		}

		if (driverList.size() <= 0) {
			String[] ps = path.split(File.pathSeparator);
			URL[] urls = new URL[ps.length];
			for (int i = 0; i < ps.length; i++) {
				urls[i] = new File(ps[i]).toURI().toURL();
			}
			if (defaultLoader == null) {
				defaultLoader = Thread.currentThread().getContextClassLoader();
				if (defaultLoader == null) {
					defaultLoader = ClassLoader.getSystemClassLoader();
				}
			}

			myLoader = new URLClassLoader(urls, defaultLoader);
			Thread.currentThread().setContextClassLoader(myLoader);

			for (URL u : myLoader.getURLs()) {
				DerbySchemaFrame.logln("URLClassLoader#url: " + u);
			}
			oldJarPath = path;

			@SuppressWarnings("unchecked")
			Iterator<Driver> i = Service.providers(Driver.class, myLoader);
			while (i.hasNext()) {
				Driver d = null;
				try {
					d = i.next();
				} catch (Throwable t) {
				}
				if (d != null) {
					DerbySchemaFrame.logln("Service.providers-> " + d);
					driverList.add(d);
				}
			}
		}
		for (Driver d : driverList) {
			Connection conn = d.connect(url, info);
			if (conn != null) {
				useDriver = d;
				return conn;
			}
		}
		throw new SQLException("No suitable driver", "08001");
	}

	static void shutdown(boolean all) throws SQLException {
		String url;
		if (all || connectUrl == null) {
			url = "jdbc:derby:"; // SĂw
		} else {
			url = connectUrl; // DBw
			int n = url.indexOf(';');
			if (n > 0) { // IvVꍇ́A
				url = url.substring(0, n);
			}
		}

		try {
			Properties info = new Properties();
			info.setProperty("shutdown", Boolean.toString(true));
			if (useDriver == null) {
				DerbySchemaFrame.logln("DriverManager.getConnection(" + url
						+ ")");
				DriverManager.getConnection(url, info);
			} else {
				DerbySchemaFrame.logln("Driver#connect(" + url + ", " + info
						+ ")");
				useDriver.connect(url, info);
			}
			// Vbg_EɐƗO
		} finally {
			connectUrl = null;
		}

		throw new SQLException("Vbg_Es[" + url + "]");
	}
}
