package jp.hishidama.javadb.tool.conn;

import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.NClob;
import java.sql.PreparedStatement;
import java.sql.SQLClientInfoException;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Savepoint;
import java.sql.Statement;
import java.sql.Struct;
import java.util.Map;
import java.util.Properties;

import jp.hishidama.javadb.tool.schema.DerbySchemaFrame;

class DelegateConnection implements Connection {

	protected Connection conn = null;

	void setConnection(Connection conn) {
		this.conn = conn;
	}

	public void clearWarnings() throws SQLException {
		DerbySchemaFrame.logln("Connection#clearWarnings()");
		conn.clearWarnings();
	}

	public void close() throws SQLException {
		DerbySchemaFrame.logln("Connection#close()");
		conn.close();
	}

	public void commit() throws SQLException {
		DerbySchemaFrame.logln("Connection#commit()");
		conn.commit();
	}

	public Array createArrayOf(String typeName, Object[] elements)
			throws SQLException {
		return conn.createArrayOf(typeName, elements);
	}

	public Blob createBlob() throws SQLException {
		return conn.createBlob();
	}

	public Clob createClob() throws SQLException {
		return conn.createClob();
	}

	public NClob createNClob() throws SQLException {
		return conn.createNClob();
	}

	public SQLXML createSQLXML() throws SQLException {
		return conn.createSQLXML();
	}

	public Statement createStatement() throws SQLException {
		return new DelegateStatement(conn.createStatement());
	}

	public Statement createStatement(int resultSetType,
			int resultSetConcurrency, int resultSetHoldability)
			throws SQLException {
		return new DelegateStatement(conn.createStatement(resultSetType,
				resultSetConcurrency, resultSetHoldability));
	}

	public Statement createStatement(int resultSetType, int resultSetConcurrency)
			throws SQLException {
		return new DelegateStatement(conn.createStatement(resultSetType,
				resultSetConcurrency));
	}

	public Struct createStruct(String typeName, Object[] attributes)
			throws SQLException {
		return conn.createStruct(typeName, attributes);
	}

	public boolean getAutoCommit() throws SQLException {
		return conn.getAutoCommit();
	}

	public String getCatalog() throws SQLException {
		return conn.getCatalog();
	}

	public Properties getClientInfo() throws SQLException {
		return conn.getClientInfo();
	}

	public String getClientInfo(String name) throws SQLException {
		return conn.getClientInfo(name);
	}

	public int getHoldability() throws SQLException {
		return conn.getHoldability();
	}

	public DatabaseMetaData getMetaData() throws SQLException {
		return conn.getMetaData();
	}

	public int getTransactionIsolation() throws SQLException {
		return conn.getTransactionIsolation();
	}

	public Map<String, Class<?>> getTypeMap() throws SQLException {
		return conn.getTypeMap();
	}

	public SQLWarning getWarnings() throws SQLException {
		return conn.getWarnings();
	}

	public boolean isClosed() throws SQLException {
		return conn.isClosed();
	}

	public boolean isReadOnly() throws SQLException {
		return conn.isReadOnly();
	}

	public boolean isValid(int timeout) throws SQLException {
		return conn.isValid(timeout);
	}

	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		return conn.isWrapperFor(iface);
	}

	public String nativeSQL(String sql) throws SQLException {
		return conn.nativeSQL(sql);
	}

	public CallableStatement prepareCall(String sql, int resultSetType,
			int resultSetConcurrency, int resultSetHoldability)
			throws SQLException {
		DerbySchemaFrame.logln("Connection#prepareCall: " + sql);
		return conn.prepareCall(sql, resultSetType, resultSetConcurrency,
				resultSetHoldability);
	}

	public CallableStatement prepareCall(String sql, int resultSetType,
			int resultSetConcurrency) throws SQLException {
		DerbySchemaFrame.logln("Connection#prepareCall: " + sql);
		return conn.prepareCall(sql, resultSetType, resultSetConcurrency);
	}

	public CallableStatement prepareCall(String sql) throws SQLException {
		DerbySchemaFrame.logln("Connection#prepareCall: " + sql);
		return conn.prepareCall(sql);
	}

	public PreparedStatement prepareStatement(String sql, int resultSetType,
			int resultSetConcurrency, int resultSetHoldability)
			throws SQLException {
		DerbySchemaFrame.logln("Connection#prepareStatement: " + sql);
		return conn.prepareStatement(sql, resultSetType, resultSetConcurrency,
				resultSetHoldability);
	}

	public PreparedStatement prepareStatement(String sql, int resultSetType,
			int resultSetConcurrency) throws SQLException {
		DerbySchemaFrame.logln("Connection#prepareStatement: " + sql);
		return conn.prepareStatement(sql, resultSetType, resultSetConcurrency);
	}

	public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys)
			throws SQLException {
		DerbySchemaFrame.logln("Connection#prepareStatement: " + sql);
		return conn.prepareStatement(sql, autoGeneratedKeys);
	}

	public PreparedStatement prepareStatement(String sql, int[] columnIndexes)
			throws SQLException {
		DerbySchemaFrame.logln("Connection#prepareStatement: " + sql);
		return conn.prepareStatement(sql, columnIndexes);
	}

	public PreparedStatement prepareStatement(String sql, String[] columnNames)
			throws SQLException {
		DerbySchemaFrame.logln("Connection#prepareStatement: " + sql);
		return conn.prepareStatement(sql, columnNames);
	}

	public PreparedStatement prepareStatement(String sql) throws SQLException {
		DerbySchemaFrame.logln("Connection#prepareStatement: " + sql);
		return conn.prepareStatement(sql);
	}

	public void releaseSavepoint(Savepoint savepoint) throws SQLException {
		DerbySchemaFrame
				.logln("Connection#releaseSavepoint(" + savepoint + ")");
		conn.releaseSavepoint(savepoint);
	}

	public void rollback() throws SQLException {
		DerbySchemaFrame.logln("Connection#rollback()");
		conn.rollback();
	}

	public void rollback(Savepoint savepoint) throws SQLException {
		DerbySchemaFrame.logln("Connection#rollback(" + savepoint + ")");
		conn.rollback(savepoint);
	}

	public void setAutoCommit(boolean autoCommit) throws SQLException {
		DerbySchemaFrame.logln("Connection#setAutoCommit(" + autoCommit + ")");
		conn.setAutoCommit(autoCommit);
	}

	public void setCatalog(String catalog) throws SQLException {
		DerbySchemaFrame.logln("Connection#setCatalog(" + catalog + ")");
		conn.setCatalog(catalog);
	}

	public void setClientInfo(Properties properties)
			throws SQLClientInfoException {
		DerbySchemaFrame.logln("Connection#setClientInfo(" + properties + ")");
		conn.setClientInfo(properties);
	}

	public void setClientInfo(String name, String value)
			throws SQLClientInfoException {
		DerbySchemaFrame.logln("Connection#setClientInfo(" + name + "=" + value
				+ ")");
		conn.setClientInfo(name, value);
	}

	public void setHoldability(int holdability) throws SQLException {
		DerbySchemaFrame
				.logln("Connection#setHoldability(" + holdability + ")");
		conn.setHoldability(holdability);
	}

	public void setReadOnly(boolean readOnly) throws SQLException {
		DerbySchemaFrame.logln("Connection#setReadOnly(" + readOnly + ")");
		conn.setReadOnly(readOnly);
	}

	public Savepoint setSavepoint() throws SQLException {
		DerbySchemaFrame.logln("Connection#setSavepoint()");
		return conn.setSavepoint();
	}

	public Savepoint setSavepoint(String name) throws SQLException {
		DerbySchemaFrame.logln("Connection#setSavepoint(" + name + ")");
		return conn.setSavepoint(name);
	}

	public void setTransactionIsolation(int level) throws SQLException {
		DerbySchemaFrame.logln("Connection#setTransactionIsolation(" + level
				+ ")");
		conn.setTransactionIsolation(level);
	}

	public void setTypeMap(Map<String, Class<?>> map) throws SQLException {
		DerbySchemaFrame.logln("Connection#setTypeMap(" + map + ")");
		conn.setTypeMap(map);
	}

	public <T> T unwrap(Class<T> iface) throws SQLException {
		DerbySchemaFrame.logln("Connection#unwrap(" + iface + ")");
		return conn.unwrap(iface);
	}
}
