package jp.hishidama.javadb.tool.data;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.TransferHandler;

import jp.hishidama.javadb.tool.schema.DerbySchemaFrame;
import jp.hishidama.swing.table.ExTable;

public class DataTransferHandler extends TransferHandler {
	private static final long serialVersionUID = -8602677489317185200L;

	@Override
	public int getSourceActions(JComponent c) {
		return COPY;
	}

	@Override
	protected Transferable createTransferable(JComponent c) {
		JTable table = (JTable) c;
		return new DataTransferable(table);
	}

	@Override
	public boolean canImport(TransferSupport support) {
		return support.isDataFlavorSupported(DataTransferable.dataFlavor)
				|| support.isDataFlavorSupported(DataFlavor.stringFlavor);
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean importData(TransferSupport support) {
		Transferable tdata = support.getTransferable();
		if (tdata.isDataFlavorSupported(DataTransferable.dataFlavor)) {
			List<List<Data>> rlist;
			try {
				rlist = (List<List<Data>>) tdata
						.getTransferData(DataTransferable.dataFlavor);
			} catch (UnsupportedFlavorException e) {
				return false;
			} catch (IOException e) {
				return false;
			}

			new Transfer<Data>() {
				@Override
				protected Data initData(Data data, Data src) {
					try {
						data.setValue(src.toString(), src.isKeyword());
					} catch (Exception e) {
						DerbySchemaFrame.log(e);
						data = null;
					}
					return data;
				}
			}.exec(support, rlist);
			return true;
		}
		if (tdata.isDataFlavorSupported(DataFlavor.stringFlavor)) {
			String str;
			try {
				str = (String) tdata.getTransferData(DataFlavor.stringFlavor);
			} catch (UnsupportedFlavorException e) {
				return false;
			} catch (IOException e) {
				return false;
			}

			String[] strr = str.split("\n");
			int max = 0;
			List<List<String>> rlist = new ArrayList<List<String>>(strr.length);
			for (int i = 0; i < strr.length; i++) {
				String[] strc = strr[i].split("\t");
				max = Math.max(max, strc.length);
				List<String> list = new ArrayList<String>(max);
				for (int j = 0; j < strc.length; j++) {
					list.add(strc[j]);
				}
				rlist.add(list);
			}

			new Transfer<String>() {
				@Override
				protected Data initData(Data data, String src) {
					try {
						data.setValue(src, false);
					} catch (Exception e) {
						DerbySchemaFrame.log(e);
						data = null;
					}
					return data;
				}
			}.exec(support, rlist);
			return true;
		}
		return false;
	}

	static abstract class Transfer<C> {
		public void exec(TransferSupport support, List<List<C>> rlist) {
			int cmax = 0;
			for (int i = 0; i < rlist.size(); i++) {
				List<C> list = rlist.get(i);
				cmax = Math.max(cmax, list.size());
			}

			JTable table = (JTable) support.getComponent();
			ExTable extable = null;
			if (table instanceof ExTable) {
				extable = (ExTable) table;
			}
			if (extable != null) {
				extable.beginUndoUpdate();
			}
			try {
				int vr, vc;
				if (support.isDrop()) {
					JTable.DropLocation loc = (JTable.DropLocation) support
							.getDropLocation();
					vr = loc.getRow();
					vc = loc.getColumn();
				} else {
					vr = table.getSelectedRow();
					vc = table.getSelectedColumn();
				}
				int[] rows = table.getSelectedRows();
				int[] cols = table.getSelectedColumns();
				List<Integer> tor = getDestIndex(vr, rows, rlist.size());
				List<Integer> toc = getDestIndex(vc, cols, cmax);

				for (int i = 0; i < rlist.size(); i++) {
					int r = tor.get(i);
					if (r >= table.getRowCount()) {
						break;
					}
					List<C> list = rlist.get(i);
					for (int j = 0; j < list.size(); j++) {
						int c = toc.get(j);
						if (c >= table.getColumnCount()) {
							break;
						}
						C d = list.get(j);
						Class<?> cls = table.getColumnClass(c);
						Data data = new Data(cls);
						data = initData(data, d);
						if (data != null) {
							table.setValueAt(data, r, c);
						}
					}
				}
			} finally {
				if (extable != null) {
					extable.endUndoUpdate();
				}
			}
		}

		protected abstract Data initData(Data data, C src);

		protected List<Integer> getDestIndex(int start, int[] sels, int size) {
			List<Integer> list = new ArrayList<Integer>(size);
			int last = start;
			list.add(last);
			int i = 1;
			for (int j = 0; j < sels.length; j++) {
				int s = sels[j];
				if (s > last) {
					last = s;
					list.add(last);
					i++;
				}
			}
			for (; i < size; i++) {
				list.add(++last);
			}
			return list;
		}
	}

}
