package jp.hishidama.javadb.tool.data;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringBufferInputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JTable;

@SuppressWarnings("deprecation")
public class DataTransferable implements Transferable {

	static class SaveData {
		Class<?> cls;
		String value;
		boolean keyword;

		SaveData(Data src) {
			if (src != null) {
				cls = src.clazz;
				value = src.toString();
				keyword = src.isKeyword();
			} else {
				cls = Object.class;
				value = null;
				keyword = false;
			}
		}
	}

	protected List<List<SaveData>> rowList;

	public DataTransferable(JTable table) {
		int[] rows = table.getSelectedRows();
		int[] cols = table.getSelectedColumns();

		rowList = new ArrayList<List<SaveData>>(rows.length);

		for (int r = 0; r < rows.length; r++) {
			List<SaveData> list = new ArrayList<SaveData>(cols.length);
			for (int c = 0; c < cols.length; c++) {
				Data value = (Data) table.getValueAt(rows[r], cols[c]);
				SaveData sdata = new SaveData(value);
				list.add(sdata);
			}
			rowList.add(list);
		}
	}

	private static DataFlavor[] dataFlavors;
	private static DataFlavor[] stringFlavors;
	private static DataFlavor[] htmlFlavors;
	private static DataFlavor[] plainFlavors;

	static {
		try {
			dataFlavors = new DataFlavor[] { new DataFlavor(Data.class,
					"HtJavaDBtoll.Data") };

			htmlFlavors = new DataFlavor[] {
					new DataFlavor("text/html;class=java.lang.String"),
					new DataFlavor("text/html;class=java.io.Reader"),
					new DataFlavor(
							"text/html;charset=unicode;class=java.io.InputStream") };

			plainFlavors = new DataFlavor[] {
					new DataFlavor("text/plain;class=java.lang.String"),
					new DataFlavor("text/plain;class=java.io.Reader"),
					new DataFlavor(
							"text/plain;charset=unicode;class=java.io.InputStream") };

			stringFlavors = new DataFlavor[] {
					new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType
							+ ";class=java.lang.String"),
					DataFlavor.stringFlavor };
		} catch (ClassNotFoundException cle) {
			System.err.println("error initializing DataTransferable");
		}
	}

	public static final DataFlavor dataFlavor = dataFlavors[0];

	@Override
	public DataFlavor[] getTransferDataFlavors() {
		DataFlavor[] fs = new DataFlavor[dataFlavors.length
				+ stringFlavors.length + htmlFlavors.length
				+ plainFlavors.length];
		int n = 0;
		n += addTransferDataFlavors(fs, dataFlavors, n);
		n += addTransferDataFlavors(fs, stringFlavors, n);
		n += addTransferDataFlavors(fs, htmlFlavors, n);
		n += addTransferDataFlavors(fs, plainFlavors, n);
		return fs;
	}

	protected static int addTransferDataFlavors(DataFlavor[] dest,
			DataFlavor[] src, int n) {
		int len = src.length;
		System.arraycopy(src, 0, dest, n, len);
		return len;
	}

	@Override
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		if (isDataFlavorSupported(flavor, dataFlavors)) {
			return true;
		}
		if (isDataFlavorSupported(flavor, stringFlavors)) {
			return true;
		}
		if (isDataFlavorSupported(flavor, htmlFlavors)) {
			return true;
		}
		if (isDataFlavorSupported(flavor, plainFlavors)) {
			return true;
		}
		return false;
	}

	protected static boolean isDataFlavorSupported(DataFlavor flavor,
			DataFlavor[] flavors) {
		for (int i = 0; i < flavors.length; i++) {
			if (flavors[i].equals(flavor)) {
				return true;
			}
		}
		return false;
	}

	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (isDataFlavorSupported(flavor, dataFlavors)) {
			List<List<Data>> rlist = new ArrayList<List<Data>>(rowList.size());
			for (int i = 0; i < rowList.size(); i++) {
				List<SaveData> slist = rowList.get(i);
				List<Data> dlist = new ArrayList<Data>(slist.size());
				for (int j = 0; j < slist.size(); j++) {
					SaveData sdata = slist.get(j);
					Data data = new Data(sdata.cls);
					try {
						data.setValue(sdata.value, sdata.keyword);
					} catch (Exception e) {
					}
					dlist.add(data);
				}
				rlist.add(dlist);
			}
			return rlist;
		}
		if (isDataFlavorSupported(flavor, stringFlavors)
				|| isDataFlavorSupported(flavor, plainFlavors)) {
			StringBuilder sb = new StringBuilder(256);
			for (int i = 0; i < rowList.size(); i++) {
				if (i != 0) {
					sb.append('\n');
				}
				List<SaveData> list = rowList.get(i);
				for (int j = 0; j < list.size(); j++) {
					if (j != 0) {
						sb.append('\t');
					}
					SaveData sdata = list.get(j);
					String str = sdata.value;
					if (str != null) {
						sb.append(str);
					}
				}
			}
			if (Reader.class.equals(flavor.getRepresentationClass())) {
				return new StringReader(sb.toString());
			} else if (InputStream.class
					.equals(flavor.getRepresentationClass())) {
				return new StringBufferInputStream(sb.toString());
			}
			return sb.toString();
		}
		if (isDataFlavorSupported(flavor, htmlFlavors)) {
			StringBuilder sb = new StringBuilder(256);
			sb.append("<html><body><table>\n");
			for (int i = 0; i < rowList.size(); i++) {
				sb.append("<tr>");
				List<SaveData> list = rowList.get(i);
				for (int j = 0; j < list.size(); j++) {
					sb.append("<td>");
					SaveData sdata = list.get(j);
					String str = sdata.value;
					if (str != null) {
						sb.append(str);
					}
					sb.append("</td>");
				}
				sb.append("</tr>\n");
			}
			sb.append("</table></body></html>");
			if (String.class.equals(flavor.getRepresentationClass())) {
				return sb.toString();
			} else if (Reader.class.equals(flavor.getRepresentationClass())) {
				return new StringReader(sb.toString());
			} else if (InputStream.class
					.equals(flavor.getRepresentationClass())) {
				return new StringBufferInputStream(sb.toString());
			}
		}
		throw new UnsupportedFlavorException(flavor);
	}
}
