package jp.hishidama.javadb.tool.free;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;
import javax.swing.table.TableRowSorter;

import jp.hishidama.javadb.tool.conn.ConnManager;
import jp.hishidama.javadb.tool.data.Data;
import jp.hishidama.javadb.tool.data.DataComparator;
import jp.hishidama.javadb.tool.schema.DerbySchemaFrame;
import jp.hishidama.swing.action.ExAction;
import jp.hishidama.swing.table.RowHeaderTable;
import jp.hishidama.swing.text.TextManager;
import jp.hishidama.swing.undo.TextUndoManager;

public class FreeSqlFrame extends JFrame {
	private static final long serialVersionUID = 1L;

	public FreeSqlFrame() {
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	}

	public void init() {
		setTitle("Free SQL");
		setSize(756, 512);
		// setLocationRelativeTo(null);
		setLocationByPlatform(true);

		Container c = getContentPane();
		c.add(new FreePanel());

		initMenuBar();
	}

	protected void initMenuBar() {

		// JMenu menu1 = new JMenu("t@C(F)");
		// menu1.setMnemonic('F');

		JMenu menu2 = new JMenu("s(E)");
		menu2.setMnemonic('E');
		{
			menu2.add(new ExecuteQueryAction());
			menu2.add(new ExecuteUpdateAction());
			menu2.add(new ExecuteAction());
			menu2.addSeparator();
			menu2.add(actionRefresh);
			menu2.addSeparator();
			menu2.add(new CommitAction());
			menu2.add(new RollbackAction());
		}

		JMenuBar bar = new JMenuBar();
		{
			// bar.add(menu1);
			bar.add(menu2);
		}

		setJMenuBar(bar);
	}

	protected JLabel sqlLabel = new JLabel();
	protected FreeDataTable table;
	protected FreeDataModel dataModel;
	protected JEditorPane editor;
	protected JSplitPane split;

	class FreePanel extends JSplitPane implements WindowFocusListener {
		private static final long serialVersionUID = -217199428449872262L;
		private boolean firstFocus;

		public FreePanel() {
			super(JSplitPane.VERTICAL_SPLIT);

			dataModel = new FreeDataModel();
			table = new FreeDataTable(dataModel);
			JScrollPane scroll = new JScrollPane(table);
			new RowHeaderTable(table, "", 48).installTo(scroll);

			JPanel up = new JPanel(new BorderLayout());
			up.add(sqlLabel, BorderLayout.NORTH);
			up.add(scroll);
			super.add(up);

			editor = new JEditorPane();
			new TextManager(new TextUndoManager()).installTo(editor);
			super.add(editor);
			Font f = editor.getFont();
			editor
					.setFont(new Font(Font.MONOSPACED, f.getStyle(), f
							.getSize()));

			super.setDividerLocation(0);

			addWindowFocusListener(this);
			firstFocus = true;

			split = this;
		}

		@Override
		public void windowGainedFocus(WindowEvent e) {
			if (firstFocus) {
				editor.requestFocusInWindow();
				firstFocus = false;
			}
		}

		@Override
		public void windowLostFocus(WindowEvent e) {
		}
	}

	public static OpenAction actionOpen = new OpenAction();

	public static class OpenAction extends ExAction {
		private static final long serialVersionUID = -2286603770191616135L;

		public OpenAction() {
			super("FreeSQL");
			putMnemonicKey('F');

			setEnabled(false);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			FreeSqlFrame frame = new FreeSqlFrame();
			frame.init();
			frame.setVisible(true);
		}
	}

	class ExecuteAction extends ExAction {
		private static final long serialVersionUID = -326222428691622107L;

		public ExecuteAction() {
			super("execute");
			putMnemonicKey('E');
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
		}

		@Override
		public void actionPerformed(ActionEvent e0) {
			String sql = getSqlString();

			try {
				Connection conn = ConnManager.getConnection();
				Statement stat = conn.createStatement();
				try {
					boolean b = stat.execute(sql);
					DerbySchemaFrame.logln("execute: " + b);

					ResultSet rs = stat.getResultSet();
					if (rs != null) {
						try {
							ResultSetMetaData md = rs.getMetaData();
							dataModel.initColumns(md);

							int r = refreshData(rs);
							DerbySchemaFrame.logln("execute: : " + r);
						} finally {
							rs.close();
						}
						setQueryString(sql);
					}
				} finally {
					stat.close();
				}
			} catch (Exception e) {
				DerbySchemaFrame.log(e);
			}
		}
	}

	class ExecuteUpdateAction extends ExAction {
		private static final long serialVersionUID = -258974595594471062L;

		public ExecuteUpdateAction() {
			super("executeUpdate");
			putMnemonicKey('U');
			putMnemonicIndexKey(7);
		}

		@Override
		public void actionPerformed(ActionEvent e0) {
			String sql = getSqlString();

			try {
				Connection conn = ConnManager.getConnection();
				Statement stat = conn.createStatement();
				try {
					int r = stat.executeUpdate(sql);
					DerbySchemaFrame.logln("executeUpdate: " + r);
				} finally {
					stat.close();
				}
			} catch (Exception e) {
				DerbySchemaFrame.log(e);
			}
		}
	}

	String querySql;

	public void setQueryString(String sql) {
		querySql = sql;
		actionRefresh.setEnabled(true);

		String s = sql.replaceAll("[ \t\r\n]+", " ");
		sqlLabel.setText(s);

		if (split.getDividerLocation() <= 48) {
			split.setDividerLocation(0.5);
		}
	}

	class ExecuteQueryAction extends ExAction {
		private static final long serialVersionUID = -4468087586134320957L;

		public ExecuteQueryAction() {
			super("executeQuery");
			putMnemonicKey('Q');
		}

		@Override
		public void actionPerformed(ActionEvent e0) {
			String sql = getSqlString();
			if (refreshData(sql, true)) {
				setQueryString(sql);
			}
		}
	}

	public String getSqlString() {
		String str = editor.getSelectedText();
		if (str == null || str.isEmpty()) {
			str = editor.getText();
		}

		if (str == null) {
			return "";
		}

		str = str.trim();
		if (str.endsWith(";")) {
			str = str.substring(0, str.length() - 1).trim();
		}

		return str;
	}

	RefreshQueryAction actionRefresh = new RefreshQueryAction();

	class RefreshQueryAction extends ExAction {
		private static final long serialVersionUID = -4271085597442121983L;

		public RefreshQueryAction() {
			super("f[^ēǍ(R)");
			putMnemonicKey('R');
			setEnabled(false);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			dataModel.setRowCount(0);
			refreshData(querySql, false);
		}
	}

	protected boolean refreshData(String sql, boolean init) {
		try {
			Connection conn = ConnManager.getConnection();
			Statement stat = conn.createStatement();
			try {
				ResultSet rs = stat.executeQuery(sql);
				try {
					if (init) {
						ResultSetMetaData md = rs.getMetaData();
						dataModel.initColumns(md);
					}

					int r = refreshData(rs);
					DerbySchemaFrame.logln("executeQuery: : " + r);
				} finally {
					rs.close();
				}
			} finally {
				stat.close();
			}
		} catch (Exception e) {
			DerbySchemaFrame.log(e);
			return false;
		}

		return true;
	}

	protected int refreshData(ResultSet rs) throws SQLException,
			ClassNotFoundException {

		int size = dataModel.getColumnCount();
		while (rs.next()) {
			Vector<Object> row = new Vector<Object>(size);
			for (int i = 1; i <= size; i++) {
				Object v = rs.getObject(i);
				Data data = newData(i - 1, v);
				row.add(data);
			}
			dataModel.addRow(row);
		}

		DataComparator comparator = new DataComparator();
		TableRowSorter<?> sorter = (TableRowSorter<?>) table.getRowSorter();
		for (int i = 0; i < size; i++) {
			sorter.setComparator(i, comparator);
		}

		return dataModel.getRowCount();
	}

	public Data newData(int mc, Object value) {
		Class<?> clz = dataModel.getColumnClass(mc);
		Data data = new Data(clz);
		if (value != null) {
			data.setValue(value);
		}
		return data;
	}

	static class CommitAction extends ExAction {
		private static final long serialVersionUID = -1057634961809054639L;

		CommitAction() {
			super("R~bg");
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_S,
					KeyEvent.CTRL_DOWN_MASK));
		}

		@Override
		public void actionPerformed(ActionEvent e0) {
			try {
				ConnManager.getConnection().commit();
			} catch (SQLException e) {
				DerbySchemaFrame.log(e);
			}
		}
	}

	static class RollbackAction extends ExAction {
		private static final long serialVersionUID = -1920306741852339652L;

		RollbackAction() {
			super("[obN");
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_R,
					KeyEvent.CTRL_DOWN_MASK));
		}

		@Override
		public void actionPerformed(ActionEvent e0) {
			try {
				ConnManager.getConnection().rollback();
			} catch (SQLException e) {
				DerbySchemaFrame.log(e);
			}
		}
	}
}
