package jp.hishidama.javadb.tool.schema;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.*;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import jp.hishidama.javadb.tool.AboutDialog;
import jp.hishidama.javadb.tool.ToolProperties;
import jp.hishidama.javadb.tool.conn.ExecuteQuery;
import jp.hishidama.javadb.tool.conn.ConnManager;
import jp.hishidama.javadb.tool.free.FreeSqlFrame;
import jp.hishidama.javadb.tool.table.DerbyTableFrame;
import jp.hishidama.javadb.tool.table.TableName;
import jp.hishidama.swing.action.ExAction;
import jp.hishidama.swing.tree.LazyTreeNode;
import jp.hishidama.swing.tree.OpenableTreeNode;

public class DerbySchemaFrame extends JFrame implements WindowListener {

	private static final long serialVersionUID = 1L;

	public static DerbySchemaFrame instance;

	public void init() {
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		initLookFeel();
		setTitle("HtJavaDB");
		getContentPane().add(new SchemaPanel());
		setSize(756, 512);
		setLocationRelativeTo(null);

		addWindowListener(this);

		try {
			// urlCombo.addItem("jdbc:derby:C:/temp/javadb/sample1");
			readDBList();
		} catch (IOException e) {
			System.err.println(e.getMessage());
		}
		ToolProperties.instance.load(PROPERTY_FILE);

		int sel = ToolProperties.instance.getDblistIndex();
		if (0 <= sel && sel < urlCombo.getItemCount()) {
			urlCombo.setSelectedIndex(sel);
		}

		initMenuBar();
	}

	protected void initLookFeel() {
		try {
			String look =
			// "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
			UIManager.getSystemLookAndFeelClassName();
			UIManager.setLookAndFeel(look);
		} catch (Exception e) {
			// ʖڂȂƂ͒߂
			e.printStackTrace();
		}
	}

	protected static final File DBLIST_FILE = new File(System
			.getProperty("user.home"), ".HtJavaDBtool.dblist.txt");
	protected static final File PROPERTY_FILE = new File(System
			.getProperty("user.home"), ".HtJavaDBtool.properties");
	public static String encoding = "MS932";

	protected void readDBList() throws IOException {
		FileInputStream is = new FileInputStream(DBLIST_FILE);
		try {
			BufferedReader r = new BufferedReader(new InputStreamReader(is,
					encoding));
			try {
				for (;;) {
					String str = r.readLine();
					if (str == null) {
						break;
					}
					str = str.trim();
					if (str.isEmpty() || str.startsWith("#")) {
						continue;
					}
					if (str.startsWith("jdbc:derby:")) {
						urlCombo.addItem(str);
					}
				}
			} finally {
				r.close();
			}
		} finally {
			is.close();
		}
	}

	protected void writeDbList() throws IOException {
		PrintStream ps = new PrintStream(DBLIST_FILE, encoding);
		try {
			int size = urlCombo.getItemCount();
			for (int i = 0; i < size; i++) {
				String str = (String) urlCombo.getItemAt(i);
				ps.println(str);
			}
		} finally {
			ps.close();
		}
	}

	@Override
	public void windowActivated(WindowEvent e) {
	}

	@Override
	public void windowClosed(WindowEvent e) {
	}

	@Override
	public void windowClosing(WindowEvent e0) {
		ConnManager.closeConnectionForce();

		try {
			writeDbList();
		} catch (IOException e) {
			e.printStackTrace();
		}

		int sel = urlCombo.getSelectedIndex();
		ToolProperties.instance.setDblistIndex(sel);

		try {
			ToolProperties.instance.store(PROPERTY_FILE);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void windowDeactivated(WindowEvent e) {
	}

	@Override
	public void windowDeiconified(WindowEvent e) {
	}

	@Override
	public void windowIconified(WindowEvent e) {
	}

	@Override
	public void windowOpened(WindowEvent e) {
	}

	private void initMenuBar() {
		JMenu menu1 = new JMenu("ڑ(C)");
		menu1.setMnemonic('C');
		{
			menu1.add(actionTreeUpdate);
			menu1.add(actionDisconnect);
			menu1.addSeparator();
			menu1.add(FreeSqlFrame.actionOpen);
		}
		JMenu menu2 = new JMenu("wv(H)");
		menu2.setMnemonic('H');
		{
			menu2.add(SettingDialog.action);
			menu2.addSeparator();
			menu2.add(AboutDialog.action);
		}

		JMenuBar bar = new JMenuBar();
		bar.add(menu1);
		bar.add(menu2);
		setJMenuBar(bar);
	}

	private JComboBox urlCombo = new JComboBox();
	{
		urlCombo.setEditable(true);
		urlCombo.registerKeyboardAction(new AbstractAction() {
			private static final long serialVersionUID = 2494310007888119249L;

			@Override
			public void actionPerformed(ActionEvent e) {
				JComboBox combo = (JComboBox) e.getSource();
				if (combo.isPopupVisible()) { // hbv_EXg\ĂƂ
					int s = combo.getSelectedIndex(); // IĂCfbNX擾
					if (s >= 0) {
						combo.removeItemAt(s); // ̃CfbNX폜
					}
				}
			}
		}, KeyStroke.getKeyStroke(KeyEvent.VK_D, KeyEvent.CTRL_DOWN_MASK),
				JComboBox.WHEN_IN_FOCUSED_WINDOW);

		Font f = urlCombo.getFont();
		urlCombo.setFont(new Font(Font.DIALOG_INPUT, Font.PLAIN, f.getSize()));
	}

	private TreeUpdateAction actionTreeUpdate = new TreeUpdateAction();

	private JButton button = new JButton(actionTreeUpdate);
	{
		Dimension sz = button.getMaximumSize();
		sz.width = Short.MAX_VALUE;
		urlCombo.setMaximumSize(sz);
	}

	private SchemaTree tree;

	private SchemaTextPane logArea;

	class SchemaPanel extends JPanel {
		private static final long serialVersionUID = -1034871966076243658L;

		public SchemaPanel() {
			super(new BorderLayout());
			// setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			JPanel p = new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
			p.add(new JLabel("URL: "));
			p.add(urlCombo);
			p.add(button);

			tree = new SchemaTree(new RootTreeNode());
			logArea = new SchemaTextPane();

			JSplitPane split = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			split.add(new JScrollPane(tree));
			split.add(new JScrollPane(logArea));
			split.setDividerSize(4);

			// add(p);
			add(p, BorderLayout.NORTH);
			add(split);
		}
	}

	class TreeUpdateAction extends ExAction {
		private static final long serialVersionUID = -8640523654050042950L;

		public TreeUpdateAction() {
			super("DBǂݍ(C)");
			putMnemonicKey('C');
		}

		@Override
		public void actionPerformed(ActionEvent e0) {
			String url = null;
			try {
				ComboBoxEditor editor = urlCombo.getEditor();
				JTextField c = (JTextField) editor.getEditorComponent();
				url = c.getText();
			} catch (Exception e) {
			}
			if (url == null) {
				url = (String) urlCombo.getSelectedItem();
			}
			if (url == null) {
				return;
			}
			url = url.trim();
			if (url.isEmpty()) {
				return;
			}
			if (urlCombo.getSelectedIndex() < 0) {
				urlCombo.addItem(url);
			}

			DefaultTreeModel model = tree.getModel();
			RootTreeNode root = (RootTreeNode) model.getRoot();
			root.setUrl(url);
			root.removeAllChildren();
			root.addChildNodes();
			model.reload();
		}
	}

	public ExAction actionDisconnect = new DisconnectAction();

	class DisconnectAction extends ExAction {
		private static final long serialVersionUID = 208530187895523416L;

		public DisconnectAction() {
			super("ؒf(D)");
			putMnemonicKey('D');

			setEnabled(false);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			ConnManager.closeConnection();
		}
	}

	public static final void log(String mes) {
		log(mes, Color.BLACK);
	}

	public static final synchronized void log(String mes, Color fc) {
		instance.logArea.append(mes, fc);
	}

	public static final void logln(String mes) {
		log(mes + "\n");
	}

	public static final void log(Exception ex) {
		if (ex instanceof SQLException) {
			for (SQLException e = (SQLException) ex; e != null; e = e
					.getNextException()) {
				String mes = String.format("%s: %s%n", e.getSQLState(), e
						.getMessage());
				log(mes, Color.RED);
			}
		} else {
			log(ex.getClass() + ": " + ex.getMessage() + "\n", Color.RED);
		}
	}

	class RootTreeNode extends LazyTreeNode {
		private static final long serialVersionUID = -5624745211791271158L;

		private String url;

		public RootTreeNode() {
			super(null);
		}

		public void setUrl(String url) {
			this.url = url;
		}

		public String toString() {
			return url;
		}

		@Override
		public void addChildNodesImpl() {
			if (url == null) {
				return;
			}
			ConnManager.openConnection(url);

			String sql = "select SCHEMAID,SCHEMANAME from SYS.SYSSCHEMAS order by SCHEMANAME";
			new ExecuteQuery() {
				@Override
				protected void setResultSet(ResultSet rs) throws SQLException {
					String id = rs.getString(1);
					String name = rs.getString(2);
					SchemaTreeNode s = new SchemaTreeNode(id, name);
					add(s);
				}
			}.start(sql);
		}
	}

	class SchemaTreeNode extends LazyTreeNode {
		private static final long serialVersionUID = -5249031313214816761L;

		private String id;

		public SchemaTreeNode(String id, String name) {
			super(name);
			this.id = id;
		}

		public String getName() {
			return (String) super.getUserObject();
		}

		@Override
		public void addChildNodesImpl() {
			String sql = "select TABLEID,TABLENAME from SYS.SYSTABLES where SCHEMAID='"
					+ id + "' order by TABLENAME";
			new ExecuteQuery() {
				@Override
				protected void setResultSet(ResultSet rs) throws SQLException {
					String tid = rs.getString(1);
					String tname = rs.getString(2);
					add(new TableTreeNode(tid, tname));
				}
			}.start(sql);
		}
	}

	class TableTreeNode extends DefaultMutableTreeNode implements
			OpenableTreeNode {
		private static final long serialVersionUID = 1008123292631206064L;
		private String id;

		public TableTreeNode(String tid, String tname) {
			super(tname);
			this.id = tid;
		}

		@Override
		public String getUserObject() {
			return (String) super.getUserObject();
		}

		@Override
		public void doOpen() {
			String table = getUserObject();
			SchemaTreeNode schema = (SchemaTreeNode) this.getParent();
			TableName dname = new TableName(schema.getName(), table, id);
			DerbyTableFrame frame = new DerbyTableFrame(DerbySchemaFrame.this,
					dname);
			frame.init();
			frame.setVisible(true);
		}
	}
}
