package jp.hishidama.javadb.tool.schema;

import java.awt.event.ActionEvent;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import jp.hishidama.javadb.tool.schema.DerbySchemaFrame.TableTreeNode;
import jp.hishidama.swing.action.ExAction;
import jp.hishidama.swing.popup.ExPopupMenu;
import jp.hishidama.swing.tree.LazyTree;

class SchemaTree extends LazyTree {
	private static final long serialVersionUID = -6376342787339757582L;

	public SchemaTree(TreeNode root) {
		super(root);
		setDragEnabled(true);

		setComponentPopupMenu(new TreePopupMenu(this));
	}

	class TreePopupMenu extends ExPopupMenu {
		private static final long serialVersionUID = -8059861853209065984L;

		public TreePopupMenu(JComponent c) {
			super(c);
		}

		@Override
		protected void addMenus() {
			addCopy();
			addSeparator();
			addMenu("J(O)", new ExpandAction(), 0, null);
		}

		@Override
		protected void refreshMenuItem(JMenuItem mi) {
			super.refreshMenuItem(mi);
			Action a = mi.getAction();
			if (a instanceof ExpandAction) {
				int row = getLeadSelectionRow();
				if (row < 0) {
					mi.setEnabled(false);
				} else {
					mi.setEnabled(true);
					if (isExpanded(row)) {
						mi.setText("(O)");
					} else {
						mi.setText("J(O)");
					}
				}
			}
		}

	}

	class ExpandAction extends ExAction {
		private static final long serialVersionUID = -6581675443252091889L;

		ExpandAction() {
			putMnemonicKey('O');
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			TreePath path = getLeadSelectionPath();
			if (path != null) {
				if (isExpanded(path)) {
					collapsePath(path);
				} else {
					Object obj = path.getLastPathComponent();
					if (obj instanceof TableTreeNode) {
						TableTreeNode node = (TableTreeNode) obj;
						node.doOpen();
					} else {
						expandPath(path);
					}
				}
			}
		}
	}
}
