package jp.hishidama.javadb.tool.schema;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.GroupLayout.ParallelGroup;
import javax.swing.GroupLayout.SequentialGroup;

import jp.hishidama.javadb.tool.ToolProperties;
import jp.hishidama.swing.radio.ExButtonGroup;
import jp.hishidama.swing.text.ExTextField;
import jp.hishidama.swing.text.VisibleCaret;

public class SettingDialog extends JDialog {
	private static final long serialVersionUID = 1449448198031081580L;

	public SettingDialog() {
		super((JDialog) null, true);
	}

	public void init() {
		setTitle("ݒ");

		initPane(getContentPane());

		pack();
		Dimension sz = getSize();
		sz.width = 480;
		setSize(sz);

		setLocationRelativeTo(null);
	}

	protected ExTextField jarPath = new ExTextField();
	ExButtonGroup driverGroup;
	protected ExTextField formatDate = new ExTextField();
	protected ExTextField formatTime = new ExTextField();
	protected ExTextField formatTimestamp = new ExTextField();
	protected Color defaultBackground = formatDate.getBackground();

	private void initPane(Container container) {
		JPanel p1 = new JPanel();
		container.add(p1);
		GroupLayout layout = new GroupLayout(p1);
		p1.setLayout(layout);

		// R|[lgm̊Ԋu󂯂ݒ
		layout.setAutoCreateGaps(true);
		layout.setAutoCreateContainerGaps(true);

		// eR|[lg̏
		JTextField file1 = new ExTextField(DerbySchemaFrame.PROPERTY_FILE
				.toString());
		JTextField file2 = new ExTextField(DerbySchemaFrame.DBLIST_FILE
				.toString());
		file1.setEditable(false);
		file2.setEditable(false);
		file1.setCaret(new VisibleCaret(file1.getCaret().getBlinkRate()));
		file2.setCaret(new VisibleCaret(file2.getCaret().getBlinkRate()));

		JRadioButton r1 = new JRadioButton("DriverManager");
		JRadioButton r2 = new JRadioButton("Driver");
		driverGroup = new ExButtonGroup(r1, r2);
		jarPath.setToolTipText("<html>̃pXςꍇAc[ċNȂƁA<br>"
				+ "ɊJĂDB̍Đڑ͏oȂB<br>"
				+ "URLClassLoadergjart@CǂݍݒĂ̂A<br>"
				+ "closeĂĂÂڑɂȂĂ܂B</html>");
		JTextField classPath = new ExTextField(System
				.getProperty("java.class.path"));
		classPath.setEditable(false);
		classPath
				.setCaret(new VisibleCaret(classPath.getCaret().getBlinkRate()));

		JComponent[][] compos = { { new JLabel("ݒt@C̏ꏊ"), file1 },
				{ null, file2 }, { new JLabel("ڑ@"), null },
				{ r1, classPath }, { r2, jarPath },
				{ new JLabel("Datȅ"), formatDate },
				{ new JLabel("Timȅ"), formatTime },
				{ new JLabel("Timestamp̏"), formatTimestamp },
		//
		};

		int ny = compos.length;
		int nx = compos[0].length;

		// ̃O[v
		{
			SequentialGroup hg = layout.createSequentialGroup();
			for (int x = 0; x < nx; x++) {
				ParallelGroup pg = layout.createParallelGroup();
				for (int y = 0; y < ny; y++) {
					JComponent c = compos[y][x];
					if (c != null) {
						pg.addComponent(c);
					}
				}
				hg.addGroup(pg);
			}
			layout.setHorizontalGroup(hg);
		}

		// ̃O[v
		{
			SequentialGroup vg = layout.createSequentialGroup();
			for (int y = 0; y < ny; y++) {
				ParallelGroup pg = layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE);
				for (int x = 0; x < nx; x++) {
					JComponent c = compos[y][x];
					if (c != null) {
						pg.addComponent(c);
					}
				}
				vg.addGroup(pg);
			}
			layout.setVerticalGroup(vg);
		}

		JPanel p2 = new JPanel(new FlowLayout());
		JButton b1 = new JButton(new OkAction());
		JButton b2 = new JButton(new CancelAction());
		{
			Dimension sz1 = b1.getPreferredSize();
			Dimension sz2 = b2.getPreferredSize();
			int w = Math.max(sz1.width, sz2.width);
			sz1.width = w;
			sz2.width = w;
			b1.setPreferredSize(sz1);
			b2.setPreferredSize(sz2);
		}
		p2.add(b1);
		p2.add(b2);
		container.add(p2, BorderLayout.SOUTH);
	}

	private void initData() {
		ToolProperties prop = ToolProperties.instance;

		int n = prop.getDriverIndex();
		driverGroup.setSelectedIndex(n);
		jarPath.setText(prop.getDriverJarPath());

		formatDate.setText(prop.getDateFormat());
		formatTime.setText(prop.getTimeFormat());
		formatTimestamp.setText(prop.getTimestampFormat());

		formatDate.setBackground(defaultBackground);
		formatTime.setBackground(defaultBackground);
		formatTimestamp.setBackground(defaultBackground);

		formatDate.clearUndoEdit();
		formatTime.clearUndoEdit();
		formatTimestamp.clearUndoEdit();
	}

	private static SettingDialog dialog;

	public static DialogAction action = new DialogAction();

	static class DialogAction extends AbstractAction {
		private static final long serialVersionUID = -4420256854766151906L;

		public DialogAction() {
			super("ݒ(S)");
			putValue(Action.MNEMONIC_KEY, KeyEvent.VK_S);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (dialog == null) {
				dialog = new SettingDialog();
				dialog.init();
			}
			dialog.initData();
			dialog.setVisible(true);
		}
	}

	class OkAction extends AbstractAction {
		private static final long serialVersionUID = -6534404016447652655L;

		public OkAction() {
			initAction(this, "ok");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			boolean err = false;

			SimpleDateFormat dateDF = checkFormat(formatDate,
					new java.sql.Date(System.currentTimeMillis()));
			if (dateDF == null) {
				err = true;
			}
			SimpleDateFormat timeDF = checkFormat(formatTime,
					new java.sql.Time(System.currentTimeMillis()));
			if (timeDF == null) {
				err = true;
			}
			SimpleDateFormat tstampDF = checkFormat(formatTimestamp,
					new java.sql.Timestamp(System.currentTimeMillis()));
			if (tstampDF == null) {
				err = true;
			}

			if (err) {
				return;
			}

			ToolProperties prop = ToolProperties.instance;
			{
				prop.setDriverIndex(driverGroup.getSelectedIndex());
				prop.setDriverJarPath(jarPath.getText());

				prop.setDateFormat(formatDate.getText(), dateDF);
				prop.setTimeFormat(formatTime.getText(), timeDF);
				prop.setTimestampFormat(formatTimestamp.getText(), tstampDF);
			}
			dialog.setVisible(false);
		}

		SimpleDateFormat checkFormat(JTextField field, Date date) {
			String format = field.getText();
			try {
				SimpleDateFormat sdf = new SimpleDateFormat(format);
				String s = sdf.format(date);
				sdf.parse(s);
				field.setBackground(defaultBackground);
				return sdf;
			} catch (Exception e) {
				field.setBackground(Color.RED);
				return null;
			}
		}
	}

	class CancelAction extends AbstractAction {
		private static final long serialVersionUID = -1548494778189717715L;

		public CancelAction() {
			initAction(this, "cancel");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			dialog.setVisible(false);
		}
	}

	public static void initAction(Action a, String key) {

		String text = UIManager.getString("OptionPane." + key + "ButtonText");
		a.putValue(Action.NAME, text);

		String num = UIManager
				.getString("OptionPane." + key + "ButtonMnemonic");
		if (num != null) {
			int n = Integer.parseInt(num);
			if (n != 0) {
				a.putValue(Action.MNEMONIC_KEY, n);
			}
		}
	}
}
