package jp.hishidama.javadb.tool.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.border.LineBorder;

import jp.hishidama.javadb.tool.data.Data;
import jp.hishidama.javadb.tool.schema.DerbySchemaFrame;
import jp.hishidama.swing.text.ExTextField;

public class DerbyGenericEditor extends DefaultCellEditor {
	private static final long serialVersionUID = 1L;

	protected JPanel panel;

	protected ExTextField text;

	protected JCheckBox check = new JCheckBox();

	protected Data data;

	protected boolean keyword;

	protected Class<?> dataClass;

	protected JFrame owner;

	protected String title;

	protected boolean dialoged;

	public DerbyGenericEditor() {
		super(new ExTextField());
		panel = new JPanel() {
			private static final long serialVersionUID = 1965725150120993841L;

			@Override
			public void requestFocus() {
				DerbyGenericEditor.this.getComponent().requestFocus();
				// DerbyGenericEditor.this.getComponent().requestFocusInWindow();
			}

			@Override
			protected boolean processKeyBinding(KeyStroke ks, KeyEvent e,
					int condition, boolean pressed) {
				if (e.getID() == KeyEvent.KEY_TYPED) {
					if (JTable.class.isInstance(e.getSource())) {
						JComponent c = DerbyGenericEditor.this.getComponent();
						if (c == text) {
							text.setText(Character.toString(e.getKeyChar()));
						} else {
							if (e.getKeyChar() == ' ') {
								check.setSelected(!check.isSelected());
							}
						}
						return true;
					}
				}
				return super.processKeyBinding(ks, e, condition, pressed);
			}
		};
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));

		text = (ExTextField) super.getComponent();
		JButton button = new JButton(new ButtonAction());
		Insets is = button.getMargin();
		is.left = is.right = 0;
		button.setMargin(is);

		Dimension sz = button.getMaximumSize();
		sz.width = Short.MAX_VALUE;
		text.setMaximumSize(sz);
		check.setMaximumSize(sz);
		check.setHorizontalAlignment(JCheckBox.CENTER);
		panel.add(text);
		panel.add(button);
	}

	@Override
	public JComponent getComponent() {
		if (data == null) {
			return (JComponent) super.getComponent();
		}
		if (!keyword && Boolean.class.isAssignableFrom(dataClass)) {
			return check;
		} else {
			return text;
		}
	}

	public void init(JFrame owner, String title, Data data) {
		this.owner = owner;
		this.title = title;

		Class<?> cls = data.getValueClass();
		if (!data.isKeyword() && Boolean.class.isAssignableFrom(cls)) {
			setClickCountToStart(1);
		} else {
			setClickCountToStart(2);
		}
	}

	public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int column) {
		data = (Data) value;
		dataClass = data.getValueClass();
		keyword = data.isKeyword();
		dialoged = false;

		check.setBackground(table.getBackground());
		if (Boolean.class.isAssignableFrom(dataClass)) {
			if (keyword) {
				check.setSelected(false);
			} else {
				Boolean b = (Boolean) data.getValue();
				check.setSelected(b);
			}
		}

		text.setBorder(new LineBorder(Color.black));
		check.setBorder(new LineBorder(Color.black));

		super.getTableCellEditorComponent(table, data, isSelected, row, column);
		changePanel();

		text.clearUndoEdit();
		if (dialog != null) {
			dialog.clearUndoEdit();
		}

		return panel;
	}

	public void changePanel() {
		JComponent c = getEditComponent(text, check);

		panel.remove(text);
		panel.remove(check);
		panel.add(c, 0);
	}

	public JComponent getEditComponent(JTextField text, JCheckBox check) {
		if (Number.class.isAssignableFrom(dataClass)) {
			text.setHorizontalAlignment(JTextField.RIGHT);
		} else {
			text.setHorizontalAlignment(JTextField.LEFT);
		}

		JComponent c;
		if (!keyword && Boolean.class.isAssignableFrom(dataClass)) {
			c = check;
		} else {
			c = text;
		}
		return c;
	}

	@Override
	public boolean stopCellEditing() {
		Data n = new Data(data.getValueClass());
		JComponent c = getComponent();
		if (keyword || c == text) {
			String str = text.getText();
			if (data.isKeyword() && "null".equals(data.getValue())) {
				if (!dialoged && !"null".equalsIgnoreCase(str)
						&& !"default".equalsIgnoreCase(str)) {
					keyword = false;
				}
			}
			try {
				n.setValue(str, keyword);
			} catch (Exception e) {
				DerbySchemaFrame.log("DerbyGenericEditor#stopCellEditing(): "
						+ dataClass + "\n", Color.RED);
				DerbySchemaFrame.log(e);
				// e.printStackTrace();
				c.setBorder(new LineBorder(Color.red));
				return false;
			}
		} else {
			n.setValue(Boolean.valueOf(check.isSelected()));
		}

		data = n;
		return super.stopCellEditing();
	}

	@Override
	public Object getCellEditorValue() {
		return data;
	}

	private DerbyDataDialog dialog;

	class ButtonAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public ButtonAction() {
			super("c");
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (dialog == null) {
				dialog = new DerbyDataDialog(owner);
			}
			dialog.init(title, DerbyGenericEditor.this);
			dialog.setVisible(true);
		}
	}
}