package jp.hishidama.javadb.tool.table;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableCellRenderer;

import jp.hishidama.javadb.tool.data.Data;

public class DerbyGenericRenderer extends DefaultTableCellRenderer {
	private static final long serialVersionUID = -1437483058849484852L;

	protected JCheckBox check = new JCheckBox();
	{
		check.setHorizontalAlignment(JCheckBox.CENTER);
		check.setBorderPainted(true);
	}

	protected Color fc, bc;

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		Data data = (Data) value;
		if (data == null) {
			data = new Data(table.getColumnClass(column));
		}

		JLabel label = (JLabel) super.getTableCellRendererComponent(table,
				data, isSelected, hasFocus, row, column);
		JComponent c;
		if (data.isKeyword()
				|| !Boolean.class.isAssignableFrom(data.getValueClass())) {
			if (Number.class.isAssignableFrom(data.getValueClass())) {
				label.setHorizontalAlignment(JLabel.RIGHT);
			} else {
				label.setHorizontalAlignment(JLabel.LEFT);
			}
			if (data.isKeyword()) {
				String str = label.getText();
				label.setText("<html><em>" + str + "</em></html>");
			}
			c = label;
		} else {
			Boolean b = (Boolean) data.getValue();
			check.setSelected(b.booleanValue());
			check.setForeground(label.getForeground());
			check.setBackground(label.getBackground());

			if (hasFocus) {
				check.setBorder(UIManager
						.getBorder("Table.focusCellHighlightBorder"));
			} else {
				check.setBorder(noFocusBorder);
			}
			c = check;
		}

		if (isSelected && bc != null) {
			Color sg = table.getSelectionBackground();
			int r = calc(bc.getRed(), sg.getRed());
			int g = calc(bc.getGreen(), sg.getGreen());
			int b = calc(bc.getBlue(), sg.getBlue());
			c.setBackground(new Color(r, g, b));
		}
		return c;
	}

	private int calc(int c1, int c2) {
		// int c = c1 * c2 / 255;
		// int c = (c1 + c2) / 2;
		// int c = (255 - c1 + c2 * 2) / 2;
		// int c = c1 * (c2 / 2 + 128) / 255;
		int c = 255 - c1 * (256 - c2) / 256;
		return c;
	}

	@Override
	public void setBackground(Color c) {
		bc = c;
		super.setBackground(c);
	}

	@Override
	public void setForeground(Color c) {
		fc = c;
		super.setForeground(c);
	}
}
