package jp.hishidama.javadb.tool.table;

import java.awt.Component;
import java.awt.Container;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import jp.hishidama.javadb.tool.schema.DerbySchemaFrame;
import jp.hishidama.swing.table.RowHeaderTable;

public class DerbyTableFrame extends JFrame {
	private static final long serialVersionUID = 7986416591114480182L;

	protected DerbySchemaFrame schemaFrame;
	protected TableName tableName;

	public DerbyTableFrame(DerbySchemaFrame parent, TableName name) {
		this.schemaFrame = parent;
		this.tableName = name;
	}

	public void init() {
		setTitle(tableName.getFullTableName());
		setSize(756, 512);
		// setLocationRelativeTo(null);
		setLocationByPlatform(true);

		Container c = getContentPane();
		c.add(new TablePanel());

		initMenuBar();
	}

	protected void initMenuBar() {

		// JMenu menu1 = new JMenu("t@C(F)");
		// menu1.setMnemonic('F');

		JMenu menu2 = new JMenu("ҏW(E)");
		menu2.setMnemonic('E');
		{
			menu2.add(dataTable.actionAddRow);
			menu2.add(dataTable.actionDelRow);
			menu2.addSeparator();
			menu2.add(dataTable.actionCommit);
			menu2.add(dataTable.actionRollback);
			menu2.addSeparator();
			menu2.add(dataTable.actionRefresh);
		}

		JMenuBar bar = new JMenuBar();
		{
			// bar.add(menu1);
			bar.add(menu2);
		}

		setJMenuBar(bar);
	}

	protected TableDataTable dataTable;

	class TablePanel extends JTabbedPane {
		private static final long serialVersionUID = 4743214453933038269L;

		public TablePanel() {
			TableDefTable defTable = new TableDefTable(tableName);
			JScrollPane defScroll = new JScrollPane(defTable);
			new RowHeaderTable(defTable, "", 48).installTo(defScroll);
			addTab("`", defScroll);
			setToolTipTextAt(0, "e[u`");

			TableKeyPane keyPane = new TableKeyPane(tableName);
			addTab("L[", keyPane);
			setToolTipTextAt(1, "vC}L[  R[h荀ځǐj");

			TableDataTable dataTable = new TableDataTable(DerbyTableFrame.this,
					defTable.getModel());
			JScrollPane dataScroll = new JScrollPane(dataTable);
			new RowHeaderTable(dataTable, "", 48).installTo(dataScroll);
			addTab("f[^", dataScroll);
			setToolTipTextAt(2, "e[ũf[^");

			setSelectedIndex(2);

			DerbyTableFrame.this.dataTable = dataTable;
		}

		@Override
		public void setSelectedIndex(int index) {
			Component c = super.getComponentAt(index);
			if (c instanceof BeforeVisible) {
				BeforeVisible b = (BeforeVisible) c;
				b.beforeVisible();
			}

			super.setSelectedIndex(index);

			if (dataTable != null) {
				if (c instanceof JScrollPane) {
					c = ((JScrollPane) c).getViewport().getView();
				}
				boolean enable = (c == dataTable);
				dataTable.actionAddRow.setEnabled(enable);
				dataTable.actionDelRow.setEnabled(enable);
				dataTable.actionCommit.setEnabled(enable);
				dataTable.actionRollback.setEnabled(enable);
				dataTable.actionRefresh.setEnabled(enable);
			}
		}
	}

	static interface BeforeVisible {
		public void beforeVisible();
	}
}
