package jp.hishidama.javadb.tool.table;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;

import javax.swing.table.DefaultTableModel;

import jp.hishidama.javadb.tool.conn.ConnManager;
import jp.hishidama.javadb.tool.schema.DerbySchemaFrame;

class TableBestModel extends DefaultTableModel {
	private static final long serialVersionUID = -7595760213860265388L;

	protected TableName tableName;

	public TableBestModel(TableName tableName) {
		this.tableName = tableName;
		addColumn("COLUMN_NAME");
		addColumn("TYPE_NAME");
		addColumn("COLUMN_SIZE");
		addColumn("DECIMAL_DIGITS");
		addColumn("PSEUDO_COLUMN");
		addColumn("SCOPE");
	}

	public void load() {
		try {
			Connection conn = ConnManager.getConnection();

			DatabaseMetaData dmd = conn.getMetaData();
			ResultSet rs = dmd.getBestRowIdentifier(null, tableName
					.getSchemaName(), tableName.getTableName(),
					DatabaseMetaData.bestRowSession, true);
			try {
				while (rs.next()) {
					String cname = rs.getString("COLUMN_NAME");
					String tname = rs.getString("TYPE_NAME");
					int csize = rs.getInt("COLUMN_SIZE");
					short digit = rs.getShort("DECIMAL_DIGITS");
					short pseud = rs.getShort("PSEUDO_COLUMN");
					short scope = rs.getShort("SCOPE");
					addRow(new Object[] { cname, tname, csize, digit,
							getPseudoString(pseud), getScopeString(scope) });
				}
			} finally {
				rs.close();
			}
		} catch (Exception e) {
			DerbySchemaFrame.log(e);
		}
	}

	public String getPseudoString(short pseudo) {
		switch (pseudo) {
		case DatabaseMetaData.bestRowUnknown:
			return "bestRowUnknown";
		case DatabaseMetaData.bestRowNotPseudo:
			return "bestRowNotPseudo";
		case DatabaseMetaData.bestRowPseudo:
			return "bestRowPseudo";
		default:
			return Short.toString(pseudo);
		}
	}

	public String getScopeString(short scope) {
		switch (scope) {
		case DatabaseMetaData.bestRowTemporary:
			return "bestRowTemporary";
		case DatabaseMetaData.bestRowTransaction:
			return "bestRowTransaction";
		case DatabaseMetaData.bestRowSession:
			return "bestRowSession";
		default:
			return Short.toString(scope);
		}
	}

	@Override
	public boolean isCellEditable(int row, int column) {
		return false;
	}
}