package jp.hishidama.javadb.tool.table;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.EventObject;

import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableRowSorter;

import jp.hishidama.javadb.tool.data.Data;
import jp.hishidama.javadb.tool.data.DataComparator;
import jp.hishidama.javadb.tool.data.DataTransferHandler;
import jp.hishidama.swing.action.ExAction;
import jp.hishidama.swing.popup.ExPopupMenu;
import jp.hishidama.swing.table.FitHeader;
import jp.hishidama.swing.table.LazyTable;

class TableDataTable extends LazyTable {
	private static final long serialVersionUID = -3005534409001275634L;

	private final DerbyTableFrame derbyTableFrame;

	public TableDataTable(DerbyTableFrame derbyTableFrame,
			TableDefModel defModel) {
		super(new TableDataModel(derbyTableFrame.tableName, defModel));
		this.derbyTableFrame = derbyTableFrame;
		setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		setAutoCreateRowSorter(true);
		setSurrendersFocusOnKeystroke(true);

		setColumnSelectionAllowed(true);
		setRowSelectionAllowed(true);

		DataComparator comparator = new DataComparator();
		TableRowSorter<?> sorter = (TableRowSorter<?>) getRowSorter();
		for (int i = 0; i < getColumnCount(); i++) {
			sorter.setComparator(i, comparator);
		}

		setTransferHandler(new DataTransferHandler());
		setDragEnabled(true);

		initPopupMenu();
	}

	@Override
	protected JTableHeader createDefaultTableHeader() {
		return new FitHeader(super.columnModel);
	}

	protected void initPopupMenu() {
		JPopupMenu pmenu = new ExPopupMenu(this) {
			private static final long serialVersionUID = -2109227206743742944L;

			@Override
			protected void addMenus() {
				addCopy();
				addPaste();
				addSeparator();
				addMenuUndo(this);
				addSeparator();
				add(actionAddRow);
				add(actionDelRow);
			}

		};
		setComponentPopupMenu(pmenu);
	}

	@Override
	public TableDataModel getModel() {
		return (TableDataModel) super.getModel();
	}

	private TableCellRenderer renderer = new DerbyGenericRenderer();

	@Override
	public TableCellRenderer getCellRenderer(int row, int column) {
		changeCellRenderer(row, column, renderer);
		return renderer;
	}

	private DerbyGenericEditor editor = new DerbyGenericEditor();

	@Override
	public TableCellEditor getCellEditor(int row, int column) {

		TableDefModel defModel = getModel().defModel;
		int mc = convertColumnIndexToModel(column);
		Object colName = defModel.getValueAt(mc, TableDefModel.COLUMN_NAME_POS);
		Object typeName = defModel.getValueAt(mc, TableDefModel.DATA_TYPE_POS);
		String title = colName + "  " + typeName;

		Data data = getValueAt(row, column);
		editor.init(derbyTableFrame, title, data);

		return editor;
	}

	AddRowAction actionAddRow;

	class AddRowAction extends LazyAddAction {
		private static final long serialVersionUID = 5043488366038456901L;

		public AddRowAction() {
			super("R[hǉ");
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0));
		}
	}

	DelRowAction actionDelRow;

	class DelRowAction extends LazyDelAction {
		private static final long serialVersionUID = 4302282547895785431L;

		public DelRowAction() {
			super("R[h폜");
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		}
	}

	CommitAction actionCommit;

	class CommitAction extends LazyCommitAction {
		private static final long serialVersionUID = -7383171549061127915L;

		public CommitAction() {
			super("XVSQLsi+R~bgj");
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_S,
					KeyEvent.CTRL_DOWN_MASK));
		}
	}

	RollbackAction actionRollback;

	class RollbackAction extends LazyRollbackAction {
		private static final long serialVersionUID = 144586106558152343L;

		public RollbackAction() {
			super("XVLZ");
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_R,
					KeyEvent.CTRL_DOWN_MASK));
		}
	}

	RefreshAction actionRefresh;

	class RefreshAction extends ExAction {
		private static final long serialVersionUID = -6250964767044824777L;

		public RefreshAction() {
			super("f[^ēǍ");
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			getModel().refreshData(false);
		}
	}

	@Override
	protected void initializeKeyAction() {
		super.initializeKeyAction();

		actionAddRow = new AddRowAction();
		actionDelRow = new DelRowAction();

		setKeyAction(actionCommit = new CommitAction());
		setKeyAction(actionRollback = new RollbackAction());
		setKeyAction(actionRefresh = new RefreshAction());
	}

	@Override
	public Data getValueAt(int row, int column) {
		return (Data) super.getValueAt(row, column);
	}

	@Override
	public boolean editCellAt(int row, int column, EventObject e) {
		boolean r = super.editCellAt(row, column, e);
		if (r) {
			if (e instanceof KeyEvent) {
				KeyEvent ke = (KeyEvent) e;
				char ch = ke.getKeyChar();
				if (!Character.isISOControl(ch)) {
					Component ec = getEditorComponent();
					if (ec instanceof JPanel) {
						JPanel p = (JPanel) ec;
						for (int i = 0; i < p.getComponentCount(); i++) {
							Component c = p.getComponent(i);
							if (c instanceof JTextField) {
								ec = c;
								break;
							}
						}
					}
					if (ec instanceof JTextField) {
						JTextField tf = (JTextField) ec;
						tf.selectAll();
						tf.requestFocusInWindow();
					}
				}
			}
		}
		return r;
	}
}