package jp.hishidama.javadb.tool.table;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.swing.table.DefaultTableModel;

import jp.hishidama.javadb.tool.conn.ExecuteQuery;

class TableDefModel extends DefaultTableModel {
	private static final long serialVersionUID = 639532346515653002L;

	static final int COLUMN_NAME_POS = 0;
	static final int DATA_TYPE_POS = 1;
	static final int DEFAULT_POS = 2;
	static final int AUTO_VALUE_POS = 3;
	static final int AUTO_START_POS = 4;
	static final int AUTO_INC_POS = 5;

	protected TableName tableName;

	public TableDefModel(TableName tableName) {
		this.tableName = tableName;
		addColumn("COLUMN_NAME");
		addColumn("DATA_TYPE");
		addColumn("DEFAULT");
		addColumn("AUTO_VALUE");
		addColumn("AUTO_START");
		addColumn("AUTO_INC");

		String sql = "select * from SYS.SYSCOLUMNS where REFERENCEID='"
				+ tableName.getTableId() + "' order by COLUMNNUMBER";
		new ExecuteQuery() {
			@Override
			protected void setResultSet(ResultSet rs) throws SQLException {
				String name = rs.getString("COLUMNNAME");
				Object type = rs.getObject("COLUMNDATATYPE");
				Object defv = rs.getObject("COLUMNDEFAULT");
				BigDecimal avalue = rs.getBigDecimal("AUTOINCREMENTVALUE");
				BigDecimal astart = rs.getBigDecimal("AUTOINCREMENTSTART");
				BigDecimal ainc = rs.getBigDecimal("AUTOINCREMENTINC");
				addRow(new Object[] { name, type, defv, avalue, astart, ainc });
			}
		}.start(sql);
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
		case COLUMN_NAME_POS:
			return String.class;
		case DATA_TYPE_POS:
		case DEFAULT_POS:
			return Object.class;
		case AUTO_VALUE_POS:
		case AUTO_START_POS:
		case AUTO_INC_POS:
			return BigDecimal.class;
		}
		return super.getColumnClass(columnIndex);
	}

	@Override
	public boolean isCellEditable(int row, int column) {
		return false;
	}
}