package jp.hishidama.swing.dialog;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.UIManager;

import jp.hishidama.swing.action.ExAction;

/**
 * vOXo[\_CAO.
 *
 * @author <a target="hishidama" href=
 *         "http://www.ne.jp/asahi/hishidama/home/tech/soft/java/swing/ProgressDialog.html"
 *         >Ђ</a>
 * @since 2009.11.01
 */
public class ProgressDialog extends JDialog implements Runnable {
	private static final long serialVersionUID = -7605680076737565525L;

	/** vOXo[ */
	protected JProgressBar progressBar;

	protected volatile boolean canceled = false;
	protected boolean autoClose = false;

	protected JButton okButton;
	protected JButton cancelButton;

	/**
	 * RXgN^[.
	 *
	 * @param owner
	 *            I[i[
	 * @param modal
	 *            [_L
	 */
	public ProgressDialog(Frame owner, boolean modal) {
		super(owner, modal);
	}

	/**
	 * vOXo[.
	 *
	 * @return vOXo[
	 */
	protected JProgressBar createProgressBar() {
		return new JProgressBar();
	}

	/**
	 * vOXo[擾.
	 *
	 * @return vOXo[
	 */
	public JProgressBar getProgressBar() {
		return progressBar;
	}

	/**
	 * vOXo[ݒlݒ.
	 *
	 * @param n
	 *            io[݂̌̒l
	 */
	public void setProgressValue(int n) {
		progressBar.setValue(n);
	}

	/**
	 * vOXo[ݒl擾.
	 *
	 * @return io[݂̌̒l
	 */
	public int getProgressValue() {
		return progressBar.getValue();
	}

	/**
	 * N[Yݒ.
	 *
	 * @param b
	 *            trueFiIɓ_CAOIɕ
	 */
	public void setAutoClose(boolean b) {
		this.autoClose = b;
	}

	/**
	 * N[Y擾.
	 *
	 * @return trueFiIɓ_CAOIɕ
	 */
	public boolean isAutoClose() {
		return autoClose;
	}

	@Override
	public void dispose() {
		canceled = true;
		super.dispose();
	}

	/**
	 * IL擾.
	 * <p>
	 * ɃLZ{^ɂăLZꂽǂ̔ɎgpB<br>
	 * _CAÔIۂtrueƂȂB
	 * </p>
	 *
	 * @return trueFiIĂ
	 */
	public boolean canceled() {
		return canceled;
	}

	/**
	 * iIݒ.
	 * <p>
	 * XbhIۂɓ\bhĂяoȂ΂ȂȂB
	 * </p>
	 */
	public synchronized void progressEnd() {
		if (progressBar.isIndeterminate()) { // sm胂[h
			progressBar.setIndeterminate(false);
		}
		if (okButton != null) {
			okButton.setEnabled(true);
		}
		if (cancelButton != null) {
			cancelButton.setEnabled(false);
		}
		if (autoClose) {
			dispose();
		}
	}

	/**
	 * _CAO.
	 */
	public void init() {
		canceled = false;
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

		Container c = getContentPane();
		initProgressPane(c);
		initCenterPane(c);
		initButtonPane(c);

		pack();
		setLocationRelativeTo(null);
	}

	protected void initProgressPane(Container c) {
		JPanel panel = new JPanel();
		progressBar = createProgressBar();
		panel.add(progressBar);
		c.add(panel, BorderLayout.PAGE_START);
	}

	protected void initCenterPane(Container c) {
	}

	protected void initButtonPane(Container c) {
		okButton = createOkButton();
		cancelButton = createCancelButton();
		if (okButton != null || cancelButton != null) {
			JPanel panel = new JPanel(new FlowLayout());
			if (okButton != null) {
				panel.add(okButton);
			}
			if (cancelButton != null) {
				panel.add(cancelButton);
			}
			c.add(panel, BorderLayout.PAGE_END);
		}
	}

	/**
	 * OK{^.
	 *
	 * @return OK{^
	 */
	protected JButton createOkButton() {
		String text = UIManager.getString("OptionPane.okButtonText");
		JButton button = new JButton(new ProgressDisposeAction(text));
		button.setEnabled(false);
		return button;
	}

	/**
	 * CANCEL{^.
	 *
	 * @return CANCEL{^
	 */
	protected JButton createCancelButton() {
		String text = UIManager.getString("OptionPane.cancelButtonText");
		JButton button = new JButton(new ProgressCancelAction(text));
		return button;
	}

	protected class ProgressDisposeAction extends ExAction {
		private static final long serialVersionUID = -1756791747373917769L;

		public ProgressDisposeAction(String name) {
			super(name);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			dispose();
		}
	}

	protected class ProgressCancelAction extends ExAction {
		private static final long serialVersionUID = -1756791747373917769L;

		public ProgressCancelAction(String name) {
			super(name);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			canceled = true;
			progressEnd();
		}
	}

	/**
	 * _CAO\ݒ.
	 * <p>
	 * trueɂƁAXbhi{@link #run()}jsJnB<br>
	 * ܂A[__CAȌꍇAXbh̏I҂B
	 * </p>
	 *
	 * @param b
	 *            trueF_CAO\
	 */
	@Override
	public void setVisible(boolean b) {
		if (b) {
			Thread thread = new Thread(this);
			thread.start();
			super.setVisible(true);
			if (isModal()) {
				try {
					thread.join();
				} catch (InterruptedException e) {
				}
			}
		} else {
			super.setVisible(false);
		}
	}

	/**
	 * Xbh.
	 * <p>
	 * \bhI[o[ChāAۂ̏sB
	 * </p>
	 *
	 * <pre>
	 * // 
	 * &#064;Override
	 * public void run() {
	 *  for (int i = 0; i &lt; 100; i++) {
	 *   if ({@link #canceled()}) {
	 *    return;
	 *   }
	 *   {@link #setProgressValue}(i + 1);
	 *  }
	 *  {@link #progressEnd()};
	 * }
	 * </pre>
	 */
	@Override
	public void run() {
		progressEnd();
	}
}
